/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.seatunnel.connectors.seatunnel.preview.config;

import org.apache.seatunnel.common.utils.JsonUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

public enum RedisDataType {
    KEY {
        @Override
        public void set(RedisClient RedisClient, String key, String value) {
            RedisClient.set(key, value);
        }

        @Override
        public List<String> get(RedisClient RedisClient, String key) {
            return Collections.singletonList(RedisClient.get(key));
        }
    },
    HASH {
        @Override
        public void set(RedisClient RedisClient, String key, String value) {
            RedisClient.hset(key, key, value);
        }

        @Override
        public List<String> get(RedisClient RedisClient, String key) {
            Map<String, String> kvMap = RedisClient.hgetAll(key);
            return Collections.singletonList(JsonUtils.toJsonString(kvMap));
        }
    },
    LIST {
        @Override
        public void set(RedisClient RedisClient, String key, String value) {
            RedisClient.lpush(key, value);
        }

        @Override
        public List<String> get(RedisClient RedisClient, String key) {
            return RedisClient.lrange(key, 0, -1);
        }
    },
    SET {
        @Override
        public void set(RedisClient RedisClient, String key, String value) {
            RedisClient.sadd(key, value);
        }

        @Override
        public List<String> get(RedisClient RedisClient, String key) {
            Set<String> members = RedisClient.smembers(key);
            return new ArrayList<>(members);
        }
    },
    ZSET {
        @Override
        public void set(RedisClient RedisClient, String key, String value) {
            RedisClient.zadd(key, 1, value);
        }

        @Override
        public List<String> get(RedisClient RedisClient, String key) {
            return RedisClient.zrange(key, 0, -1);
        }
    };

    public List<String> get(RedisClient RedisClient, String key) {
        return Collections.emptyList();
    }

    public void set(RedisClient RedisClient, String key, String value) {
        // do nothing
    }
}
